/* FOR COMPILATION AND DEBUGGING PURPOSES ONLY! */
/* This fe.c file contains stubs for all FE routines used by the universe.
 * This file is a quick-and-dirty text based FE, provided to allow us to
 * test our functions without waiting for FE to compile our code with theirs.  
 * A more sophisticated front end has been written by the FE group, although
 * due to its nature (machine and compiler dependent code), not everyone can 
 * compile their version. */

#include <stdio.h> /* For stderr */
#include <string.h>
#include <malloc.h>

#include "main.h"        /* For Button definitions */
#include "untypes.h"
#include "fe.h"
#include "ungame.h" /* for FE_Update's output */

/*****************************
 * Function Definitions
 ****************************/

void *FE_malloc(unsigned long mem)
{
    char *ptr = (char *) malloc(mem);
    if (ptr == NULL) {
        FE_DisplayError(E_MEMORY);
    }
    return (void *) ptr;
}

void FE_free(void *ptr)
{
    free(ptr);
}

/* Action keys on the left of a QWERTY keyboard; system keys on the right */
UN_Actions FE_GetKeys()
{
    char in;
    scanf("%c\n",&in);
    printf("Input read: %c\n",in);
    if (in == 'w') return (UN_Actions) ACT_THRUST;
    else if (in == 'x') return (UN_Actions) ACT_BRAKE;
    else if (in == 'a') return (UN_Actions) ACT_LEFT;
    else if (in == 'd') return (UN_Actions) ACT_RIGHT;
    else if (in == 's') return (UN_Actions) ACT_FIRE;
    else if (in == 'q') return (UN_Actions) ACT_NEXT_WEAPON;
    else if (in == 'e') return (UN_Actions) ACT_PREV_WEAPON;
    else if (in == 'z') return (UN_Actions) ACT_NEXT_TARGET;
    else if (in == 'c') return (UN_Actions) ACT_PREV_TARGET;
    else if (in == 'r') return (UN_Actions) ACT_EAT;
    else if (in == 'v') return (UN_Actions) ACT_BREED;
    else if (in == 'p') return (UN_Actions) SYS_PAUSE;
    else if (in == ';') return (UN_Actions) SYS_QUIT;
    else if (in == 'l') return (UN_Actions) SYS_END_GAME;
    else if (in == ',') return (UN_Actions) SYS_TOGGLE_MUSIC;
    else if (in == '.') return (UN_Actions) SYS_TOGGLE_SFX;
    else return (UN_Actions) 0;
}

FE_Ticks FE_Update(UN_Object* **Sectors,
                   UN_Coord centerX,
                   UN_Coord centerY,
                   unsigned short shiftsX,
                   unsigned short shiftsY)
{
   UN_Object *current;
   printf("New frame:\n");
   for (current = ActiveObjects->next_glob; /* skip dummy */ 
            current != NULL; /* as long as we are not at the end */
            current = current->next_glob) /* next object */
     {
        if (current->what == OBJ_PLAYER) printf("PLAYER:");
        else if (current->what == OBJ_PROBE) printf("PROBE:");
        else if (current->what == OBJ_ASTEROID) printf("ROCK:");
        else if (current->what == OBJ_BULLET) printf("BULLET:");
/* Anything else (like food) has no header in front of its position */
        printf("X:%i Y:%i VELX:%f VELY:%f FACING:%i SHIELD:%i ENERGY:%i\n",
            current->x,current->y,current->velx,current->vely,current->facedir,
            current->shield,current->energy);
     }
   return (FE_Ticks) 0;
}

char *FE_CompileSprite(char *source)
{
    return source;
}

void FE_PointerStatus(short* x, short* y, char* b1, char* b2)
{
    static char ans[80];
    printf("Which button do you want the pointer on?\n");
    printf("NEW, REP, LOAD, CONT, INST, SAVE, OPT, CRED, QUIT, YES, NO\n");
    scanf("%s",ans);
    if ((ans[0] == 'N') && (ans[1] == 'E')) {
        *x = NEW_btn.ulx; *y = NEW_btn.uly;
    } else if (ans[0] == 'R') {
        *x = REP_btn.ulx; *y = REP_btn.uly;
    } else if (ans[0] == 'L') {
        *x = LOAD_btn.ulx; *y = LOAD_btn.uly;
    } else if ((ans[0] == 'C') && (ans[0] == 'O')) {
        *x = CONT_btn.ulx; *y = CONT_btn.uly;
    } else if (ans[0] == 'I') {
        *x = INST_btn.ulx; *y = INST_btn.uly;
    } else if (ans[0] == 'S') {
        *x = SAVE_btn.ulx; *y = SAVE_btn.uly;
    } else if (ans[0] == 'O') {
        *x = OPT_btn.ulx; *y = OPT_btn.uly;
    } else if ((ans[0] == 'C') && (ans[1] == 'R')) {
        *x = CRED_btn.ulx; *y = CRED_btn.uly;
    } else if (ans[0] == 'Q') {
        *x = QUIT_btn.ulx; *y = QUIT_btn.uly;
    } else if (ans[0] == 'Y') {
        *x = YES_btn.ulx; *y = YES_btn.uly;
    } else if ((ans[0] == 'N') && (ans[1] == 'O')) {
        *x = NO_btn.ulx; *y = NO_btn.uly;
    } else {
        *x = 0; *y = 0;
    }
    *x += 1; *y += 1; /* So it's inside the button box */
    *b1 = 1;
    *b2 = 1;
}

void FE_DisplayError(FE_ErrorNum err)
{
    static char* errorStrs[] = {
        "Hey! This should never appear, coz error 0 is not an error!",
        "Yikes!"
        "MEMORY ERROR"
    };
    fprintf(stderr,"%s\n",errorStrs[err]);
}

void FE_DisplayScreen(FE_ScreenNum screenNum)
{
    switch (screenNum) {
        case MAINMENU:
            printf("Main menu\n");
            break;
        case PAUSEMENU:
            printf("Pause menu\n");
            break;
        case CONFIRMQUITDIALOG:
            printf("Confirm quit dialog\n");
            break;
        case SAVEGAMESCREEN:
        case LOADGAMESCREEN:
            printf("Save or load game screen\n");
            break;
        case OPTIONSCREEN:
            printf("Option screen\n");
            break;
        default:
            printf("Save or load slot screen\n");
            break;
    }
}

void FE_DeInitSystem()
{
}

FE_Ticks FE_Set()
{
    return (FE_Ticks) 0;
}

FE_Ticks FE_GetTicks(FE_Ticks nextUpdate)
{
    return nextUpdate;
}

FE_ErrorNum FE_LoadGame(short slot)
{
    return E_NOERROR;
}

FE_ErrorNum FE_SaveGame(short slot)
{
    return E_NOERROR;
}

void FE_ToggleSound()
{
}

FE_ErrorNum FE_InitSystem(int *argc_ret, char *argv[])
{
    printf("%s initialized.\n",argv[0]);
    return (FE_ErrorNum) 0;
}
